@@ -119,11 +119,14 @@ be displayed as:
 .. autosummary::
 
    ~fsleyes.gl.glvolume.GLVolume
+   ~fsleyes.gl.glrgbvolume.GLRGBVolume
+   ~fsleyes.gl.glcomplex.GLComplex
    ~fsleyes.gl.glmask.GLMask
    ~fsleyes.gl.gllabel.GLLabel
    ~fsleyes.gl.gllinevector.GLLineVector
    ~fsleyes.gl.glrgbvector.GLRGBVector
    ~fsleyes.gl.glmesh.GLMesh
+   ~fsleyes.gl.glmip.GLMIP
    ~fsleyes.gl.gltensor.GLTensor
    ~fsleyes.gl.glsh.GLSH
 
@@ -255,6 +258,9 @@ def bootstrap(glVersion=None):
     ``glvolume_funcs``     The version-specific module containing functions for
                            rendering :class:`.GLVolume` instances.
 
+    ``glrgbvolume_funcs``  The version-specific module containing functions for
+                           rendering :class:`.GLRGBVolume` instances.
+
     ``glrgbvector_funcs``  The version-specific module containing functions for
                            rendering :class:`.GLRGBVector` instances.
 
@@ -275,6 +281,9 @@ def bootstrap(glVersion=None):
 
     ``glsh_funcs``         The version-specific module containing functions for
                            rendering :class:`.GLSH` instances.
+
+    ``glmip_funcs``        The version-specific module containing functions for
+                           rendering :class:`.GLMIP` instances.
     ====================== ====================================================
 
 
@@ -300,7 +309,7 @@ def bootstrap(glVersion=None):
         return
 
     if glVersion is None:
-        glVer        = gl.glGetString(gl.GL_VERSION).decode('ascii').split()[0]
+        glVer = gl.glGetString(gl.GL_VERSION).decode('latin1').split()[0]
         major, minor = [int(v) for v in glVer.split('.')][:2]
     else:
         major, minor = glVersion
@@ -352,15 +361,17 @@ def bootstrap(glVersion=None):
                                'cannot run on the available graphics '
                                'hardware.'.format(', '.join(exts)))
 
-        # Tensor/SH overlays are not available in GL14
+        # Tensor/SH/MIP overlays are not available in GL14
         import fsleyes.displaycontext as dc
         dc.ALL_OVERLAY_TYPES            .remove('tensor')
         dc.ALL_OVERLAY_TYPES            .remove('sh')
+        dc.ALL_OVERLAY_TYPES            .remove('mip')
         dc.OVERLAY_TYPES['DTIFitTensor'].remove('tensor')
         dc.OVERLAY_TYPES['Image']       .remove('sh')
         dc.OVERLAY_TYPES['Image']       .remove('tensor')
+        dc.OVERLAY_TYPES['Image']       .remove('mip')
 
-    renderer = gl.glGetString(gl.GL_RENDERER).decode('ascii')
+    renderer = gl.glGetString(gl.GL_RENDERER).decode('latin1')
     log.debug('Using OpenGL {} implementation with renderer {}'.format(
         verstr, renderer))
 
@@ -390,6 +401,7 @@ def bootstrap(glVersion=None):
     thismod.GL_VERSION         = verstr
     thismod.GL_RENDERER        = renderer
     thismod.glvolume_funcs     = glpkg.glvolume_funcs
+    thismod.glrgbvolume_funcs  = glpkg.glrgbvolume_funcs
     thismod.glrgbvector_funcs  = glpkg.glrgbvector_funcs
     thismod.gllinevector_funcs = glpkg.gllinevector_funcs
     thismod.glmask_funcs       = glpkg.glmask_funcs
@@ -397,6 +409,7 @@ def bootstrap(glVersion=None):
     thismod.gllabel_funcs      = glpkg.gllabel_funcs
     thismod.gltensor_funcs     = glpkg.gltensor_funcs
     thismod.glsh_funcs         = glpkg.glsh_funcs
+    thismod.glmip_funcs        = glpkg.glmip_funcs
     thismod._bootstrapped      = True
     fslplatform.glVersion      = thismod.GL_VERSION
     fslplatform.glRenderer     = thismod.GL_RENDERER
@@ -418,7 +431,7 @@ def bootstrap(glVersion=None):
         dc.SceneOpts.performance.setAttribute(None, 'default', 1)
 
 
-def getGLContext(*args, **kwargs):
+def getGLContext(**kwargs):
     """Create and return a GL context object for on- or off-screen OpenGL
     rendering.
 
@@ -438,9 +451,16 @@ def getGLContext(*args, **kwargs):
 
     # A context has already been created
     if hasattr(thismod, '_glContext'):
+
+        # If a callback was provided,
+        # make sure it gets called.
+        callback = kwargs.pop('ready', None)
+        if callback is not None:
+            idle.idle(callback)
+
         return thismod._glContext
 
-    thismod._glContext = GLContext(*args, **kwargs)
+    thismod._glContext = GLContext(**kwargs)
 
     return thismod._glContext
 
@@ -505,26 +525,31 @@ class GLContext(object):
                  other=None,
                  target=None,
                  createApp=False,
-                 ready=None):
+                 ready=None,
+                 raiseErrors=False):
         """Create a ``GLContext``.
 
-        :arg offscreen: On-screen or off-screen context?
+        :arg offscreen:   On-screen or off-screen context?
 
-        :arg parent:    Parent ``wx`` GUI object
+        :arg parent:      Parent ``wx`` GUI object
 
-        :arg other:     Another ``GLContext`` instance with which GL state
-                        should be shared.
+        :arg other:       Another ``GLContext`` instance with which GL state
+                          should be shared.
 
-        :arg target:    If ``other`` is not ``None``, this must be a reference
-                        to a ``WXGLCanvasTarget``, the rendering target for the
-                        new context.
+        :arg target:      If ``other`` is not ``None``, this must be a
+                          reference to a ``WXGLCanvasTarget``, the rendering
+                          target for the new context.
 
-        :arg createApp: If ``True``, and if possible, this ``GLContext`` will
-                        create and run a ``wx.App`` so that it can create a
-                        ``wx.glcanvas.GLContext``.
+        :arg createApp:   If ``True``, and if possible, this ``GLContext`` will
+                          create and run a ``wx.App`` so that it can create a
+                          ``wx.glcanvas.GLContext``.
 
-        :arg ready:     Function which will be called when the context has
-                        been created and is ready to use.
+        :arg ready:       Function which will be called when the context has
+                          been created and is ready to use.
+
+        :are raiseErrors: Defaults to ``False``. If ``True``, and if the
+                          ``ready`` function raises an error, that error is not
+                          caught.
         """
 
         def defaultReady():
@@ -545,8 +570,8 @@ class GLContext(object):
         haveGui          = fslplatform.haveGui
 
         # On-screen contexts *must* be
-        # created on the wx.MainLoop
-        if (not offscreen) and (not haveGui):
+        # created via a wx event loop
+        if (not offscreen) and not (haveGui or createApp):
             raise ValueError('On-screen GL contexts must be '
                              'created on the wx.MainLoop')
 
@@ -601,6 +626,8 @@ class GLContext(object):
                                     '{}: {}'.format(type(e).__name__,
                                                     str(e)),
                                                     exc_info=True)
+                        if raiseErrors:
+                            raise e
 
                 # Destroying the dummy canvas
                 # can be dangerous on GTK, so we
@@ -633,12 +660,20 @@ class GLContext(object):
                 log.debug('Starting temporary wx.MainLoop')
                 self.__app.MainLoop()
 
+                # Once the GL context has been
+                # created, we no longer need
+                # references to the wx objects
+                self.__parent = None
+                self.__canvas = None
+                self.__app    = None
+
 
     def setTarget(self, target):
         """Set the given ``WXGLCanvasTarget`` or ``OffScreenCanvasTarget`` as
         the target for GL rendering with this context.
         """
-        if not self.__offscreen:
+        import wx.glcanvas as wxgl
+        if not self.__offscreen and isinstance(target, wxgl.GLCanvas):
             self.__context.SetCurrent(target)
 
 
@@ -729,7 +764,7 @@ class GLContext(object):
             # until the dummy canvas is
             # physically shown on the screen.
             while not self.__canvas.IsShownOnScreen():
-                wx.Yield()
+                wx.GetApp().Yield()
 
             self.__context.SetCurrent(self.__canvas)
 
@@ -849,11 +884,10 @@ class OffScreenCanvasTarget(object):
 
         self._setGLContext()
         self._initGL()
-        self.__target.setSize(self.__width, self.__height)
+        self.__target.shape = self.__width, self.__height
 
-        self.__target.bindAsRenderTarget()
-        self._draw()
-        self.__target.unbindAsRenderTarget()
+        with self.__target.target():
+            self._draw()
 
 
     def getBitmap(self):
@@ -864,7 +898,7 @@ class OffScreenCanvasTarget(object):
         """
 
         self._setGLContext()
-        return self.__target.getData()
+        return self.__target.getBitmap()
 
 
     def saveToFile(self, filename):
@@ -928,9 +962,18 @@ class WXGLCanvasTarget(object):
         self.__glReady           = False
         self.__freezeDraw        = False
         self.__freezeSwapBuffers = False
-        self.__dpiscale          = 1.0
         self.__context           = context
 
+        # Under macOS, high DPI must be explicitly
+        # requested for GL canvases. This can be
+        # done via the EnableHighDPI method. Under
+        # Linux, GL canvases are scaled the same as
+        # other windows.
+        if platform.system() == 'Darwin':
+            self.__dpiscale = 1.0
+        else:
+            self.__dpiscale = self.GetContentScaleFactor()
+
         self.Bind(wx.EVT_PAINT,            self.__onPaint)
         self.Bind(wx.EVT_ERASE_BACKGROUND, self.__onEraseBackground)
 
@@ -1038,8 +1081,10 @@ class WXGLCanvasTarget(object):
             # Just in case this canvas
             # was destroyed before it
             # had a chance to be drawn
-            # on
-            except wx.PyDeadObjectError:
+            # on - wxpython raises a
+            # RuntimeError on attempts
+            # to use deleted wx objects
+            except RuntimeError:
                 pass
 
         if not self.__glReady:
@@ -1053,7 +1098,8 @@ class WXGLCanvasTarget(object):
         This method should be called before any OpenGL operations related to
         this canvas take place (e.g. texture/data creation, drawing, etc).
         """
-        if not self.IsShownOnScreen(): return False
+        if not (fwidgets.isalive(self) and self.IsShownOnScreen()):
+            return False
 
         log.debug('Setting context target to {} ({})'.format(
             type(self).__name__, id(self)))
@@ -1067,15 +1113,23 @@ class WXGLCanvasTarget(object):
         return self.GetClientSize().Get()
 
 
+    def GetContentScaleFactor(self):
+        """Overrides ``wx.Window.GetContentScaleFactor``.
+
+        Calls the base class implementation, except where wxpython 3.0.2.0 is
+        being used, as the method does not exist in that version. In this
+        case, 1.0 is returned.
+        """
+        try:
+            return float(super().GetContentScaleFactor())
+        except AttributeError:
+            return 1.0
+
+
     def GetScale(self):
         """Returns the current DPI scaling factor. """
 
-        # GetContentScaleFactor does not
-        # exist in wxpython 3.0.2.0
-        try:
-            scale = float(self.GetContentScaleFactor())
-        except AttributeError:
-            scale = 1.0
+        scale = self.GetContentScaleFactor()
 
         # Reset scaling factor in case the canvas
         # window has moved between displays with
@@ -1138,34 +1192,33 @@ class WXGLCanvasTarget(object):
         """Attempts to enable/disable high-resolution rendering.
         """
 
+        # Not relevant under linux -
+        # see note in __init__
+        if platform.system() != 'Darwin':
+            return
+
         if not self._setGLContext():
             return
 
         self.__dpiscale = 1.0
 
-        # GetContentScaleFactor does not
-        # exist in wxpython 3.0.2.0
-        try:
-            scale = self.GetContentScaleFactor()
-
-        except AttributeError:
-            return
-
         # If the display can't scale,
         # (scale == 1) there's no point
         # in enabling it.
-        scale  = float(scale)
+        scale  = self.GetContentScaleFactor()
         enable = enable and scale > 1
 
-        # TODO Support other platforms
+        # on macOS, we have to set
+        # scaling on the GL canvas
         try:
             import objc
+            nsview = objc.objc_object(c_void_p=self.GetHandle())
+            nsview.setWantsBestResolutionOpenGLSurface_(enable)
+
+        # objc library not present
         except ImportError:
             return
 
-        nsview = objc.objc_object(c_void_p=self.GetHandle())
-        nsview.setWantsBestResolutionOpenGLSurface_(enable)
-
         if enable: self.__dpiscale = scale
         else:      self.__dpiscale = 1.0
 
