@@ -16,10 +16,11 @@ import numpy                              as np
 import OpenGL.GL                          as gl
 
 from   fsl.utils.platform import platform as fslplatform
-import fsl.utils.async                    as async
-import fsl.utils.transform                as transform
+import fsl.utils.idle                     as idle
+import fsl.transform.affine               as affine
 import fsleyes.gl                         as fslgl
 import fsleyes.gl.routines                as glroutines
+import fsleyes.gl.shaders.filter          as glfilter
 from . import                                textures
 from . import                                glimageobject
 from . import resources                   as glresources
@@ -114,11 +115,17 @@ class GLVolume(glimageobject.GLImageObject):
     from the 'camera' through the image texture. The resulting colour
     is generated from sampling points along the ray.
 
+    The ``glvolume_funcs`` modules are expected to perform this rendering
+    off-screen, using two :class:`.RenderTexture` instances, available as
+    attributes ``renderTexture1`` and``renderTexture2``.  After a call to
+    ``draw3D``, the final result is assuemd to be contained in
+    ``renderTexture1``.
+
 
     **Textures**
 
 
-    The ``GLVolume`` class uses three textures:
+    The ``GLVolume`` class uses the following textures:
 
      - An :class:`.ImageTexture`, a 3D texture which contains image data.
        This is bound to texture unit 0.
@@ -137,6 +144,11 @@ class GLVolume(glimageobject.GLImageObject):
        texture will not be bound - in this case, the image texture is used
        for clipping.
 
+     - Two :class:`.RenderTexture` instances which are used for 3D rendering.
+       Both of these textures have depth buffers. When one of these textures
+       is being drawn it is bound to texture units 4 (for RGBA) and 5 (for
+       depth).
+
 
     **Attributes**
 
@@ -153,6 +165,10 @@ class GLVolume(glimageobject.GLImageObject):
                          colour map.
     ``negColourTexture`` The :class:`.ColourMapTexture` used to store the
                          negative colour map.
+    ``renderTexture1``   The first :class:`.RenderTexture` used for 3D
+                         rendering.
+    ``renderTexture2``   The first :class:`.RenderTexture` used for 3D
+                         rendering.
     ``texName``          A name used for the ``imageTexture``,
                          ``colourTexture``, and ``negColourTexture`. The
                          name for the latter is suffixed with ``'_neg'``.
@@ -160,22 +176,24 @@ class GLVolume(glimageobject.GLImageObject):
     """
 
 
-    def __init__(self, image, displayCtx, canvas, threedee):
+    def __init__(self, image, overlayList, displayCtx, canvas, threedee):
         """Create a ``GLVolume`` object.
 
         :arg image:       An :class:`.Image` object.
 
+        :arg overlayList: The :class:`.OverlayList`
+
         :arg displayCtx:  The :class:`.DisplayContext` object managing the
                           scene.
 
         :arg canvas:      The canvas doing the drawing.
 
         :arg threedee:    Set up for 2D or 3D rendering.
-
         """
 
         glimageobject.GLImageObject.__init__(self,
                                              image,
+                                             overlayList,
                                              displayCtx,
                                              canvas,
                                              threedee)
@@ -196,7 +214,7 @@ class GLVolume(glimageobject.GLImageObject):
         # Ref to an OpenGL shader program -
         # the glvolume_funcs module will
         # create this for us.
-        self.shader   = None
+        self.shader = None
 
         # References to the clip image and
         # associated DisplayOpts instance,
@@ -212,10 +230,14 @@ class GLVolume(glimageobject.GLImageObject):
             '{}_neg'.format(self.texName))
 
         if self.threedee:
+
+            self.smoothFilter = glfilter.Filter('smooth', texture=0)
+            self.smoothFilter.set(kernSize=self.opts.smoothing * 2)
+
             self.renderTexture1 = textures.RenderTexture(
-                self.name, gl.GL_LINEAR, rttype='cd')
+                self.name, interp=gl.GL_LINEAR, rttype='cd')
             self.renderTexture2 = textures.RenderTexture(
-                self.name, gl.GL_LINEAR, rttype='cd')
+                self.name, interp=gl.GL_LINEAR, rttype='cd')
 
         # This attribute is used by the
         # updateShaderState method to
@@ -237,10 +259,15 @@ class GLVolume(glimageobject.GLImageObject):
         # Call glvolume_funcs.init when the image
         # and clip textures are ready to be used.
         def init():
-            fslgl.glvolume_funcs.init(self)
-            self.notify()
 
-        async.idleWhen(init, self.texturesReady)
+            # In some crazy circumstances,
+            # a just-created GLVolume can
+            # get destroyed immediately
+            if not self.destroyed:
+                fslgl.glvolume_funcs.init(self)
+                self.notify()
+
+        idle.idleWhen(init, self.texturesReady)
 
 
     def destroy(self):
@@ -254,11 +281,11 @@ class GLVolume(glimageobject.GLImageObject):
         self.removeDisplayListeners()
 
         self.imageTexture.deregister(self.name)
-        glresources.delete(self.imageTexture.getTextureName())
+        glresources.delete(self.imageTexture.name)
 
         if self.clipTexture is not None:
             self.clipTexture.deregister(self.name)
-            glresources.delete(self.clipTexture.getTextureName())
+            glresources.delete(self.clipTexture.name)
 
         self.colourTexture   .destroy()
         self.negColourTexture.destroy()
@@ -271,8 +298,10 @@ class GLVolume(glimageobject.GLImageObject):
         if self.threedee:
             self.renderTexture1.destroy()
             self.renderTexture2.destroy()
+            self.smoothFilter  .destroy()
             self.renderTexture1 = None
             self.renderTexture2 = None
+            self.smoothFilter   = None
 
         fslgl.glvolume_funcs       .destroy(self)
         glimageobject.GLImageObject.destroy(self)
@@ -282,7 +311,7 @@ class GLVolume(glimageobject.GLImageObject):
         """Returns ``True`` if this ``GLVolume`` is ready to be drawn,
         ``False`` otherwise.
         """
-        return (not self.destroyed()    and
+        return (not self.destroyed      and
                 self.shader is not None and
                 self.texturesReady())
 
@@ -329,6 +358,7 @@ class GLVolume(glimageobject.GLImageObject):
         opts    .addListener('invertClipping',   name,
                              self._invertClippingChanged)
         opts    .addListener('cmap',             name, self._cmapChanged)
+        opts    .addListener('gamma',            name, self._cmapChanged)
         opts    .addListener('interpolateCmaps', name, self._cmapChanged)
         opts    .addListener('negativeCmap',     name, self._cmapChanged)
         opts    .addListener('cmapResolution',   name, self._cmapChanged)
@@ -336,6 +366,7 @@ class GLVolume(glimageobject.GLImageObject):
                              self._useNegativeCmapChanged)
         opts    .addListener('invert',           name, self._invertChanged)
         opts    .addListener('volume',           name, self._volumeChanged)
+        opts    .addListener('channel',          name, self._channelChanged)
         opts    .addListener('interpolation',    name,
                              self._interpolationChanged)
         opts    .addListener('transform',        name, self._transformChanged)
@@ -349,23 +380,24 @@ class GLVolume(glimageobject.GLImageObject):
         # 3D-only options
         if self.threedee:
 
-            opts.addListener('dithering',       name, self._ditheringChanged)
             opts.addListener('numSteps',        name, self._numStepsChanged)
             opts.addListener('numInnerSteps',   name,
                              self._numInnerStepsChanged)
-            opts.addListener('resolution',   name,    self._resolutionChanged)
+            opts.addListener('resolution',      name,  self._resolutionChanged)
             opts.addListener('blendFactor',     name, self._blendFactorChanged)
+            opts.addListener('smoothing',       name, self._smoothingChanged)
             opts.addListener('showClipPlanes',  name,
                              self._showClipPlanesChanged)
             opts.addListener('numClipPlanes',
                              name,
                              self._numClipPlanesChanged)
+            opts.addListener('clipMode',        name, self._clipModeChanged)
             opts.addListener('clipPosition',    name, self._clipping3DChanged)
             opts.addListener('clipAzimuth',     name, self._clipping3DChanged)
             opts.addListener('clipInclination', name, self._clipping3DChanged)
 
         # GLVolume instances need to keep track of whether
-        # the volume property of their corresponding
+        # the volume/channel properties of their corresponding
         # VolumeOpts instance is synced to other VolumeOpts
         # instances - if it is, there an ImageTexture for
         # the image may already exist (i.e. have been
@@ -386,6 +418,9 @@ class GLVolume(glimageobject.GLImageObject):
             opts.addSyncChangeListener('volume',
                                        name,
                                        self._imageSyncChanged)
+            opts.addSyncChangeListener('channel',
+                                       name,
+                                       self._imageSyncChanged)
 
 
     def removeDisplayListeners(self):
@@ -405,12 +440,14 @@ class GLVolume(glimageobject.GLImageObject):
         opts    .removeListener(          'clipImage',               name)
         opts    .removeListener(          'invertClipping',          name)
         opts    .removeListener(          'cmap',                    name)
+        opts    .removeListener(          'gamma',                   name)
         opts    .removeListener(          'interpolateCmaps',        name)
         opts    .removeListener(          'negativeCmap',            name)
         opts    .removeListener(          'useNegativeCmap',         name)
         opts    .removeListener(          'cmapResolution',          name)
         opts    .removeListener(          'invert',                  name)
         opts    .removeListener(          'volume',                  name)
+        opts    .removeListener(          'channel',                 name)
         opts    .removeListener(          'interpolation',           name)
         opts    .removeListener(          'transform',               name)
         opts    .removeListener(          'displayXform',            name)
@@ -418,16 +455,21 @@ class GLVolume(glimageobject.GLImageObject):
         opts    .removeListener(          'overrideDataRange',       name)
 
         if self.threedee:
-            opts.removeListener('dithering',       name)
             opts.removeListener('numSteps',        name)
-            opts.removeListener('numClipPlanes',   name)
+            opts.removeListener('numInnerSteps',   name)
+            opts.removeListener('resolution',      name)
+            opts.removeListener('blendFactor',     name)
+            opts.removeListener('smoothing',       name)
             opts.removeListener('showClipPlanes',  name)
+            opts.removeListener('numClipPlanes',   name)
+            opts.removeListener('clipMode',        name)
             opts.removeListener('clipPosition',    name)
             opts.removeListener('clipAzimuth',     name)
             opts.removeListener('clipInclination', name)
 
         if self.__syncListenersRegistered:
-            opts.removeSyncChangeListener('volume', name)
+            opts.removeSyncChangeListener('volume',  name)
+            opts.removeSyncChangeListener('channel', name)
 
 
     def testUnsynced(self):
@@ -438,16 +480,18 @@ class GLVolume(glimageobject.GLImageObject):
         ``GLVolume`` instance needs to create its own image texture;
         returns ``False`` otherwise.
         """
-        is4D = len(self.image.shape) >= 4 and self.image.shape[3] > 1
+        is4D  = len(self.image.shape) >= 4 and self.image.shape[3] > 1
+        isRGB = self.image.nvals > 1
 
-        return (self.opts.getParent() is None or
-                (is4D and not self.opts.isSyncedToParent('volume')))
+        return ((self.opts.getParent() is None)                      or
+                (is4D  and not self.opts.isSyncedToParent('volume')) or
+                (isRGB and not self.opts.isSyncedToParent('channel')))
 
 
     def updateShaderState(self, *args, **kwargs):
         """Calls :func:`.gl14.glvolume_funcs.updateShaderState` or
         :func:`.gl21.glvolume_funcs.updateShaderStatea`, then
-        :meth:`.Notifier.notify`. Uses the :func:`.async.idleWhen` function to
+        :meth:`.Notifier.notify`. Uses the :func:`.idle.idleWhen` function to
         make sure that it is not called until :meth:`ready` returns ``True``.
 
         :arg alwaysNotify: Must be passed as a keyword argument. If
@@ -492,17 +536,20 @@ class GLVolume(glimageobject.GLImageObject):
         # single event, but in this situation
         # we only want to actually do the
         # update once.
-        async.idleWhen(func,
-                       self.ready,
-                       name=self.name,
-                       skipIfQueued=True)
+        idle.idleWhen(func,
+                      self.ready,
+                      name=self.name,
+                      skipIfQueued=True)
 
 
-    def refreshImageTexture(self):
+    def refreshImageTexture(self, **kwargs):
         """Refreshes the :class:`.ImageTexture` used to store the
         :class:`.Image` data. This is performed through the :mod:`.resources`
         module, so the image texture can be shared between multiple
         ``GLVolume`` instances.
+
+        All keyword arguments are passed through to the :class:`.ImageTexture`
+        constructor.
         """
 
         opts     = self.opts
@@ -514,23 +561,29 @@ class GLVolume(glimageobject.GLImageObject):
 
         if self.imageTexture is not None:
 
-            if self.imageTexture.getTextureName() == texName:
+            if self.imageTexture.name == texName:
                 return None
 
             self.imageTexture.deregister(self.name)
-            glresources.delete(self.imageTexture.getTextureName())
+            glresources.delete(self.imageTexture.name)
 
         if opts.interpolation == 'none': interp = gl.GL_NEAREST
         else:                            interp = gl.GL_LINEAR
 
+        if opts.enableOverrideDataRange: normRange = opts.overrideDataRange
+        else:                            normRange = None
+
         self.imageTexture = glresources.get(
             texName,
-            textures.ImageTexture,
+            textures.createImageTexture,
             texName,
             self.image,
             interp=interp,
+            channel=opts.channel,
             volume=opts.index()[3:],
-            notify=False)
+            normaliseRange=normRange,
+            notify=False,
+            **kwargs)
 
         self.imageTexture.register(self.name, self.__texturesChanged)
 
@@ -587,7 +640,7 @@ class GLVolume(glimageobject.GLImageObject):
 
         if self.clipTexture is not None:
             self.clipTexture.deregister(self.name)
-            glresources.delete(self.clipTexture.getTextureName())
+            glresources.delete(self.clipTexture.name)
             self.clipTexture = None
 
         if clipImage is None:
@@ -619,6 +672,7 @@ class GLVolume(glimageobject.GLImageObject):
         cmap    = opts.cmap
         interp  = opts.interpolateCmaps
         res     = opts.cmapResolution
+        gamma   = opts.realGamma(opts.gamma)
         negCmap = opts.negativeCmap
         invert  = opts.invert
         dmin    = opts.displayRange[0]
@@ -631,6 +685,7 @@ class GLVolume(glimageobject.GLImageObject):
                                invert=invert,
                                alpha=alpha,
                                resolution=res,
+                               gamma=gamma,
                                interp=interp,
                                displayRange=(dmin, dmax))
 
@@ -638,6 +693,7 @@ class GLVolume(glimageobject.GLImageObject):
                                   invert=invert,
                                   alpha=alpha,
                                   resolution=res,
+                                  gamma=gamma,
                                   interp=interp,
                                   displayRange=(dmin, dmax))
 
@@ -673,39 +729,49 @@ class GLVolume(glimageobject.GLImageObject):
         """Calls the version dependent ``draw3D`` function. """
 
         opts = self.opts
-        w, h = self.canvas.GetSize()
+        w, h = self.canvas.GetScaledSize()
         res  = self.opts.resolution / 100.0
-        w    = int(np.ceil(w * res))
-        h    = int(np.ceil(h * res))
+        sw   = int(np.ceil(w * res))
+        sh   = int(np.ceil(h * res))
 
         # Initialise and resize
         # the offscreen textures
         for rt in [self.renderTexture1, self.renderTexture2]:
-            if rt.getSize() != (w, h):
-                rt.setSize(w, h)
+            if rt.shape != (sw, sh):
+                rt.shape = sw, sh
 
-            rt.bindAsRenderTarget()
-            gl.glClearColor(0, 0, 0, 0)
-            gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
-            rt.unbindAsRenderTarget()
+            with rt.target():
+                gl.glClearColor(0, 0, 0, 0)
+                gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
 
         if opts.resolution != 100:
-            gl.glViewport(0, 0, w, h)
+            gl.glViewport(0, 0, sw, sh)
 
         # Do the render. Even though we're
         # drawing off-screen,  we need to
         # enable depth-testing, otherwise
         # depth values will not get written
         # to the depth buffer!
+        #
+        # The glvolume_funcs.draw3D function
+        # will put the final render into
+        # renderTexture1
         with glroutines.enabled((gl.GL_DEPTH_TEST, gl.GL_CULL_FACE)):
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_FILL)
             gl.glFrontFace(gl.GL_CCW)
             gl.glCullFace(gl.GL_BACK)
             fslgl.glvolume_funcs.draw3D(self, *args, **kwargs)
 
-        # renderTexture1 should now
-        # contain the final result -
-        # draw it to the screen.
+        # Apply smoothing if needed. If smoothing
+        # is enabled, the final final render will
+        # be in renderTexture2
+        if opts.smoothing > 0:
+            self.smoothFilter.set(offsets=[1.0 / sw, 1.0 / sh])
+            self.smoothFilter.osApply(self.renderTexture1,
+                                      self.renderTexture2)
+
+        # We now have the final result
+        # - draw it to the screen.
         verts = np.array([[-1, -1, 0],
                           [-1,  1, 0],
                           [ 1, -1, 0],
@@ -713,15 +779,32 @@ class GLVolume(glimageobject.GLImageObject):
                           [-1,  1, 0],
                           [ 1,  1, 0]], dtype=np.float32)
 
-        invproj = transform.invert(self.canvas.getProjectionMatrix())
-        verts   = transform.transform(verts, invproj)
+        invproj = affine.invert(self.canvas.projectionMatrix)
+        verts   = affine.transform(verts, invproj)
 
         if opts.resolution != 100:
-            w, h = self.canvas.GetSize()
             gl.glViewport(0, 0, w, h)
 
-        with glroutines.enabled((gl.GL_DEPTH_TEST)):
-            self.renderTexture1.draw(verts, useDepth=True)
+        with glroutines.enabled(gl.GL_DEPTH_TEST):
+
+            # If smoothing was not applied, rt1
+            # contains the final render. Otherwise,
+            # rt2 contains the final render, but rt1
+            # contains the depth information. So we
+            # need to # temporarily replace rt2.depth
+            # with rt1.depth.
+            if opts.smoothing > 0:
+                src    = self.renderTexture2
+                olddep = self.renderTexture2.depthTexture
+                dep    = self.renderTexture1.depthTexture
+            else:
+                src    = self.renderTexture1
+                olddep = self.renderTexture1.depthTexture
+                dep    = olddep
+
+            src.depthTexture = dep
+            src.draw(verts, useDepth=True)
+            src.depthTexture = olddep
 
 
     def drawAll(self, *args, **kwargs):
@@ -757,13 +840,47 @@ class GLVolume(glimageobject.GLImageObject):
         if self.opts.clipImage is None:
             clipCoordXform = np.eye(4)
         else:
-            clipCoordXform = transform.concat(
+            clipCoordXform = affine.concat(
                 self.clipOpts.getTransform('display', 'texture'),
-                self.opts    .getTransform('texture', 'display'))
+                self.opts    .getTransform('texture', 'display'),
+                self.imageTexture.invTexCoordXform(self.overlay.shape))
 
         return clipCoordXform
 
 
+    def generateVertices2D(self, zpos, axes, bbox=None):
+        """Overrides :meth:`.GLImageObject.generateVertices2D`.
+
+        Appliies the :meth:`.ImageTextureBase.texCoordXform` to the texture
+        coordinates - this is performed to support 2D images/textures.
+        """
+
+        vertices, voxCoords, texCoords = \
+            glimageobject.GLImageObject.generateVertices2D(
+                self, zpos, axes, bbox)
+
+        texCoords = affine.transform(
+            texCoords, self.imageTexture.texCoordXform(self.overlay.shape))
+
+        return vertices, voxCoords, texCoords
+
+
+    def generateVertices3D(self, bbox=None):
+        """Overrides :meth:`.GLImageObject.generateVertices3D`.
+
+        Appliies the :meth:`.ImageTextureBase.texCoordXform` to the texture
+        coordinates - this is performed to support 2D images/textures.
+        """
+
+        vertices, voxCoords, texCoords = \
+            glimageobject.GLImageObject.generateVertices3D(self, bbox)
+
+        texCoords = affine.transform(
+            texCoords, self.imageTexture.texCoordXform(self.overlay.shape))
+
+        return vertices, voxCoords, texCoords
+
+
     def _alphaChanged(self, *a):
         """Called when the :attr:`.Display.alpha` property changes. """
 
@@ -843,7 +960,7 @@ class GLVolume(glimageobject.GLImageObject):
         """Called when the :attr:`.VolumeOpts.enableOverrideDataRange` property
         changes. Calls :meth:`_volumeChanged`.
         """
-        self._volumeChanged()
+        self._volumeChanged(volRefresh=True)
 
 
     def _overrideDataRangeChanged(self, *a):
@@ -852,7 +969,7 @@ class GLVolume(glimageobject.GLImageObject):
         :attr:`.VolumeOpts.enableOverrideDataRange` is ``True``.
         """
         if self.opts.enableOverrideDataRange:
-            self._volumeChanged()
+            self._volumeChanged(volRefresh=True)
 
 
     def _volumeChanged(self, *a, **kwa):
@@ -869,6 +986,7 @@ class GLVolume(glimageobject.GLImageObject):
         else:                            normRange = None
 
         self.imageTexture.set(volume=opts.index()[3:],
+                              channel=opts.channel,
                               interp=interp,
                               volRefresh=volRefresh,
                               normaliseRange=normRange)
@@ -877,6 +995,13 @@ class GLVolume(glimageobject.GLImageObject):
             self.clipTexture.set(interp=interp)
 
 
+    def _channelChanged(self, *a, **kwa):
+        """Called when the :attr:`.NiftiOpts.channel` changes.
+        Refreshes the texture.
+        """
+        self._volumeChanged()
+
+
     def _interpolationChanged(self, *a):
         """Called when the :attr:`.NiftiOpts.interpolation` property changes.
         """
@@ -895,12 +1020,6 @@ class GLVolume(glimageobject.GLImageObject):
         self.notify()
 
 
-    def _ditheringChanged(self, *a):
-        """Called when the :attr:`.Volume3DOpts.dithering` property changes.
-        """
-        self.notify()
-
-
     def _numStepsChanged(self, *a):
         """Called when the :attr:`.Volume3DOpts.numSteps` property changes.
         """
@@ -931,6 +1050,15 @@ class GLVolume(glimageobject.GLImageObject):
         self.updateShaderState(alwaysNotify=True)
 
 
+    def _clipModeChanged(self, *a):
+        """Called when the :attr:`.Volume3DOpts.clipMode` property
+        changes.
+        """
+        if float(fslplatform.glVersion) == 1.4:
+            fslgl.glvolume_funcs.compileShaders(self)
+        self.updateShaderState(alwaysNotify=True)
+
+
     def _clipping3DChanged(self, *a):
         """Called when any of the :attr:`.Volume3DOpts.clipPosition`,
         :attr:`.Volume3DOpts.clipAzimuth`, or
@@ -954,6 +1082,14 @@ class GLVolume(glimageobject.GLImageObject):
         self.updateShaderState(alwaysNotify=True)
 
 
+    def _smoothingChanged(self, *a):
+        """Called when the :attr:`.Volume3DOpts.smoothing` property
+        changes.
+        """
+        self.smoothFilter.set(kernSize=self.opts.smoothing * 2)
+        self.notify()
+
+
     def _imageSyncChanged(self, *a):
         """Called when the synchronisation state of the
         :attr:`.NiftiOpts.volume` or :attr:`.VolumeOpts.interpolation`
