@@ -10,12 +10,11 @@ class for all panels which display overlays using ``OpenGL``.
 
 
 import logging
-import deprecation
 
 import wx
 
 import fsl.utils.idle                              as idle
-from   fsl.utils.platform import platform          as fslplatform
+import fsl.utils.deprecated                        as deprecated
 import fsleyes_props                               as props
 
 import fsleyes.actions                             as actions
@@ -30,6 +29,7 @@ import fsleyes.controls.locationpanel              as locationpanel
 import fsleyes.controls.clusterpanel               as clusterpanel
 import fsleyes.controls.lookuptablepanel           as lookuptablepanel
 import fsleyes.controls.melodicclassificationpanel as melclasspanel
+import fsleyes.controls.filetreepanel              as filetreepanel
 from . import                                         colourbarpanel
 from . import                                         viewpanel
 
@@ -95,6 +95,7 @@ class CanvasPanel(viewpanel.ViewPanel):
        toggleClusterPanel
        toggleLookupTablePanel
        toggleClassificationPanel
+       toggleFileTreePanel
 
 
     .. _canvaspanel-adding-content:
@@ -221,6 +222,12 @@ class CanvasPanel(viewpanel.ViewPanel):
     """
 
 
+    movieSyncRefresh = props.Boolean(default=True)
+    """Whether, when in movie mode, to synchronise the refresh for GL
+    canvases. This is not possible in some platforms/environments.
+    """
+
+
     def __init__(self, parent, overlayList, displayCtx, frame, sceneOpts):
         """Create a ``CanvasPanel``.
 
@@ -252,9 +259,6 @@ class CanvasPanel(viewpanel.ViewPanel):
             self.bindProps('syncLocation',
                            displayCtx,
                            displayCtx.getSyncPropertyName('worldLocation'))
-            self.bindProps('syncLocation',
-                           displayCtx,
-                           displayCtx.getSyncPropertyName('vertexIndex'))
             self.bindProps('syncOverlayOrder',
                            displayCtx,
                            displayCtx.getSyncPropertyName('overlayOrder'))
@@ -278,6 +282,8 @@ class CanvasPanel(viewpanel.ViewPanel):
         self.__movieGifAction = moviegif.MovieGifAction(
             overlayList, displayCtx, self)
 
+        self.bindProps('movieSyncRefresh', sceneOpts)
+
         self.toggleMovieMode  .bindProps('toggled', self, 'movieMode')
         self.toggleDisplaySync.bindProps('toggled', self, 'syncOverlayDisplay')
         self.toggleVolumeSync .bindProps('toggled', self, 'syncOverlayVolume')
@@ -322,6 +328,10 @@ class CanvasPanel(viewpanel.ViewPanel):
         self.__opts.addListener('fgColour',
                                 self.__name,
                                 self.__bgfgColourChanged)
+        self.__opts.addListener('labelSize',
+                                self.__name,
+                                self.__labelSizeChanged)
+
 
         idle.idle(self.__bgfgColourChanged)
 
@@ -342,6 +352,7 @@ class CanvasPanel(viewpanel.ViewPanel):
         self.sceneOpts  .removeListener('showColourBar',     self.__name)
         self.sceneOpts  .removeListener('bgColour',          self.__name)
         self.sceneOpts  .removeListener('fgColour',          self.__name)
+        self.sceneOpts  .removeListener('labelSize',         self.__name)
         self.__movieGifAction.destroy()
 
         self.__opts           = None
@@ -510,6 +521,14 @@ class CanvasPanel(viewpanel.ViewPanel):
                          canvasPanel=self)
 
 
+    @actions.toggleControlAction(filetreepanel.FileTreePanel)
+    def toggleFileTreePanel(self):
+        """Toggles a :class:`.FileTreePanel`. See
+        :meth:`.ViewPanel.togglePanel`.
+        """
+        self.togglePanel(filetreepanel.FileTreePanel, location=wx.RIGHT)
+
+
     @property
     def sceneOpts(self):
         """Returns the :class:`.SceneOpts` instance used by this
@@ -549,9 +568,7 @@ class CanvasPanel(viewpanel.ViewPanel):
         return None
 
 
-    @deprecation.deprecated(deprecated_in='0.16.0',
-                            removed_in='1.0.0',
-                            details='Use sceneOpts instead')
+    @deprecated.deprecated('0.16.0', '1.0.0', 'Use sceneOpts instead')
     def getSceneOptions(self):
         """Returns the :class:`.SceneOpts` instance used by this
         ``CanvasPanel``.
@@ -559,9 +576,7 @@ class CanvasPanel(viewpanel.ViewPanel):
         return self.__opts
 
 
-    @deprecation.deprecated(deprecated_in='0.16.0',
-                            removed_in='1.0.0',
-                            details='Use contentPanel instead')
+    @deprecated.deprecated('0.16.0', '1.0.0', 'Use contentPanel instead')
     def getContentPanel(self):
         """Returns the ``wx.Panel`` to which sub-classes must add their content.
         See the note on :ref:`adding content <canvaspanel-adding-content>`.
@@ -569,9 +584,7 @@ class CanvasPanel(viewpanel.ViewPanel):
         return self.__contentPanel
 
 
-    @deprecation.deprecated(deprecated_in='0.16.0',
-                            removed_in='1.0.0',
-                            details='Use containerPanel instead')
+    @deprecated.deprecated('0.16.0', '1.0.0', 'Use containerPanel instead')
     def getContainerPanel(self):
         """Returns the ``wx.Panel`` which contains the
         :class:`.ColourBarPanel` if it is being displayed, and the content
@@ -581,9 +594,7 @@ class CanvasPanel(viewpanel.ViewPanel):
         return self.__containerPanel
 
 
-    @deprecation.deprecated(deprecated_in='0.16.0',
-                            removed_in='1.0.0',
-                            details='Use colourBarCanvas instead')
+    @deprecated.deprecated('0.16.0', '1.0.0', 'Use colourBarCanvas instead')
     def getColourBarCanvas(self):
         """If a colour bar is being displayed, this method returns
         the :class:`.ColourBarCanvas` instance which is used by the
@@ -614,6 +625,10 @@ class CanvasPanel(viewpanel.ViewPanel):
 
         self.layoutContainerPanel()
 
+        sizer = self.centrePanel.GetSizer()
+        if sizer is not None:
+            sizer.Clear()
+
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(self.__containerPanel, flag=wx.EXPAND, proportion=1)
         self.centrePanel.SetSizer(sizer)
@@ -629,6 +644,11 @@ class CanvasPanel(viewpanel.ViewPanel):
         and is available for custom sub-class implementations to use.
         """
 
+        sizer = self.__containerPanel.GetSizer()
+        if sizer is not None:
+            sizer.Clear()
+            sizer = None
+
         sopts = self.sceneOpts
 
         if not sopts.showColourBar:
@@ -637,7 +657,11 @@ class CanvasPanel(viewpanel.ViewPanel):
                 sopts.unbindProps('colourBarLabelSide',
                                   self.__colourBar.colourBar,
                                   'labelSide')
+                sopts.unbindProps('colourBarSize',
+                                  self.__colourBar.canvas,
+                                  'barSize')
                 sopts.unbindProps('highDpi', self.__colourBar.canvas)
+
                 self.__colourBar.destroy()
                 self.__colourBar.Destroy()
                 self.__colourBar = None
@@ -656,13 +680,18 @@ class CanvasPanel(viewpanel.ViewPanel):
 
             bg = sopts.bgColour
             fg = sopts.fgColour
+            fs = sopts.labelSize
             self.__colourBar.colourBar.textColour = fg
             self.__colourBar.colourBar.bgColour   = bg
+            self.__colourBar.colourBar.fontSize   = fs
 
-        sopts.bindProps('colourBarLabelSide',
-                        self.__colourBar.colourBar,
-                        'labelSide')
-        sopts.bindProps('highDpi', self.__colourBar.canvas)
+            sopts.bindProps('colourBarLabelSide',
+                            self.__colourBar.colourBar,
+                            'labelSide')
+            sopts.bindProps('colourBarSize',
+                            self.__colourBar.canvas,
+                            'barSize')
+            sopts.bindProps('highDpi', self.__colourBar.canvas)
 
         if   sopts.colourBarLocation in ('top', 'bottom'):
             self.__colourBar.colourBar.orientation = 'horizontal'
@@ -691,6 +720,17 @@ class CanvasPanel(viewpanel.ViewPanel):
         self.centrePanelLayout()
 
 
+    def __labelSizeChanged(self, *a, **kwa):
+        """Called when the :class:`.SceneOpts.lablSize` changes.  If a colour
+        bar is being displayed, it is updated, and the panel layout is
+        refreshed.
+        """
+        sopts = self.sceneOpts
+        if self.__colourBar is not None:
+            self.__colourBar.canvas.colourBar.fontSize = sopts.labelSize
+        wx.CallAfter(self.Layout)
+
+
     def __bgfgColourChanged(self, *a, **kwa):
         """Called when the :class:`.SceneOpts.bgColour` or
         :class:`.SceneOpts.fgColour` properties change.  Updates
@@ -703,6 +743,10 @@ class CanvasPanel(viewpanel.ViewPanel):
         :arg refresh: Must be passed as a keyword argument. If ``True`` (the
                       default), this ``OrthoPanel`` is refreshed.
         """
+
+        if self.destroyed:
+            return
+
         refresh = kwa.pop('refresh', True)
 
         sceneOpts = self.sceneOpts
@@ -711,8 +755,8 @@ class CanvasPanel(viewpanel.ViewPanel):
         bg        = sceneOpts.bgColour
         fg        = sceneOpts.fgColour
 
-        cpanel.SetBackgroundColour([c * 255 for c in bg])
-        cpanel.SetForegroundColour([c * 255 for c in fg])
+        cpanel.SetBackgroundColour([round(c * 255) for c in bg])
+        cpanel.SetForegroundColour([round(c * 255) for c in fg])
 
         if self.__colourBar is not None:
             canvas = self.__colourBar.canvas
@@ -736,9 +780,9 @@ class CanvasPanel(viewpanel.ViewPanel):
         # cause delays in frame updates. So when
         # movie mode is on, we bump up the rate.
         def startMovie():
-            idle.setIdleTimeout(10)
+            idle.idleLoop.callRate = 10
             if not self.__movieLoop(startLoop=True):
-                idle.setIdleTimeout(None)
+                idle.idleLoop.callRate = None
 
         # The __movieModeChanged method is called
         # on the props event queue. Here we make
@@ -746,7 +790,7 @@ class CanvasPanel(viewpanel.ViewPanel):
         # the props event queue, by calling it from
         # the idle loop.
         if self.movieMode: idle.idle(startMovie)
-        else:              idle.setIdleTimeout(None)
+        else:              idle.idleLoop.callRate = None
 
 
     def __movieLoop(self, startLoop=False):
@@ -936,7 +980,7 @@ class CanvasPanel(viewpanel.ViewPanel):
 
         from . import scene3dpanel
 
-        if self.destroyed():   return False
+        if self.destroyed:     return False
         if not self.movieMode: return False
 
         overlay  = self.displayCtx.getSelectedOverlay()
@@ -990,72 +1034,44 @@ class CanvasPanel(viewpanel.ViewPanel):
 
         rate = (rateMin + (rateMax - rate)) / 1000.0
 
-        # The canvas refreshes are performed by the
-        # __syncMovieRefresh or __unsyncMovieRefresh
-        # methods. Gallium seems to have a problem
-        # with separate renders/buffer swaps, so we
-        # have to use a shitty unsynchronised update
-        # routine.
-        #
-        # TODO Ideally, figure out a refresh
-        #      regime that works across all
-        #      drivers. Failing this, make
-        #      this switch user controllable.
-        renderer = fslplatform.glRenderer.lower()
-        unsyncRenderers = ['gallium', 'mesa dri intel(r)']
-        useSync = not any([r in renderer for r in unsyncRenderers])
-
-        if useSync: update = self.__syncMovieRefresh
-        else:       update = self.__unsyncMovieRefresh
+        def update():
+            self.movieSync()
+            idle.idle(self.__movieLoop, after=rate)
 
         # Refresh the canvases when all
         # GLObjects are ready to be drawn.
-        idle.idleWhen(update, allReady, canvases, rate, pollTime=rate / 10)
+        idle.idleWhen(update, allReady, pollTime=rate / 10)
 
         return True
 
 
-    def __unsyncMovieRefresh(self, canvases, rate):
-        """Called by :meth:`__movieUpdate`. Updates all canvases in an
-        unsynchronised manner.
+    def movieSync(self):
+        """Called by :meth:`__movieUpdate`. Updates all GL canvases, attempting
+        to refresh them in a synchronised manner.
 
         Ideally all canvases should be drawn off-screen (i.e. rendered to the
         back buffer), and then all refreshed together (back and front buffers
         swapped). Unfortunately some OpenGL drivers seem to have trouble with
         this approach, and require drawing and front/back buffer swaps to be
-        done at the same time. This method is used for those drivers.
+        done at the same time.
 
-        :arg canvases: List of canvases to update. It is assumed that
-                       ``FreezeDraw`` and ``FreezeSwapBuffers`` has been
-                       called on every canvas.
-        :arg rate:     Delay to trigger the next movie update.
+        This method will refresh the GL canvases in either a synchronised or
+        unsynchronised manner, depending upon the value of the
+        :attr:`movieSyncRefresh` property.
         """
 
-        for c in canvases:
-            c.ThawDraw()
-            c.ThawSwapBuffers()
-            c.Refresh()
+        canvases = self.getGLCanvases()
 
-        idle.idle(self.__movieLoop, after=rate)
+        if self.movieSyncRefresh:
+            for c in canvases:
+                c.ThawDraw()
+                c.Refresh()
 
-
-    def __syncMovieRefresh(self, canvases, rate):
-        """Updates all canvases in a synchronised manner. All canvases are
-        refreshed, and then the front/back buffers are swapped on each of
-        them.
-
-        :arg canvases: List of canvases to update. It is assumed that
-                       ``FreezeDraw`` and ``FreezeSwapBuffers`` has been
-                       called on every canvas.
-        :arg rate:     Delay to trigger the next movie update.
-        """
-
-        for c in canvases:
-            c.ThawDraw()
-            c.Refresh()
-
-        for c in canvases:
-            c.ThawSwapBuffers()
-            c.SwapBuffers()
-
-        idle.idle(self.__movieLoop, after=rate)
+            for c in canvases:
+                c.ThawSwapBuffers()
+                c.SwapBuffers()
+        else:
+            for c in canvases:
+                c.ThawDraw()
+                c.ThawSwapBuffers()
+                c.Refresh()
